@@ -23,6 +23,7 @@ THE SOFTWARE.
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include "libxfontint.h"
 #include <string.h>
 
 #include <X11/fonts/fntfilst.h>
@@ -89,9 +90,7 @@ FTPickMapping(char *xlfd, int length, char *filename, FT_Face face,
            currently only work for BDFs. */
         if(strlen(enc) + strlen(reg) > 18)
             goto native;
-        strcpy(buf, enc);
-        strcat(buf, "-");
-        strcat(buf, reg);
+        snprintf(buf, sizeof(buf), "%s-%s", enc, reg);
         ErrorF("%s %s\n", buf, encoding_name);
         if(strcasecmp(buf, "iso10646-1") != 0) {
             if(strcasecmp(buf, encoding_name) == 0)
