@@ -19,10 +19,12 @@
  * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  * PERFORMANCE OF THIS SOFTWARE.
  */
- 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#define LIBXFONT_SKIP_ERRORF
+#include "libxfontint.h"
 #define FONT_t
 #define TRANS_CLIENT
 #include <X11/Xtrans/transport.c>
