@@ -34,6 +34,7 @@ from The Open Group.
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include "libxfontint.h"
 #include <X11/Xos.h>
 #include <X11/fonts/fontmisc.h>
 #include <X11/fonts/bufio.h>
@@ -82,9 +83,9 @@ BufFileRawFill (BufFilePtr f)
 static int
 BufFileRawSkip (BufFilePtr f, int count)
 {
-    int	    curoff;
-    int	    fileoff;
-    int	    todo;
+    off_t	curoff;
+    off_t	fileoff;
+    off_t	todo;
 
     curoff = f->bufp - f->buffer;
     fileoff = curoff + f->left;
@@ -93,7 +94,7 @@ BufFileRawSkip (BufFilePtr f, int count)
 	f->left -= count;
     } else {
 	todo = count - (fileoff - curoff);
-	if (lseek (FileDes(f), todo, 1) == -1) {
+	if (lseek (FileDes(f), todo, SEEK_CUR) == -1) {
 	    if (errno != ESPIPE)
 		return BUFFILEEOF;
 	    while (todo) {
@@ -162,8 +163,10 @@ BufFileOpenWrite (int fd)
     setmode(fd,O_BINARY);
 #endif
     f = BufFileCreate ((char *)(long) fd, 0, BufFileRawFlush, 0, BufFileFlush);
-    f->bufp = f->buffer;
-    f->left = BUFFILESIZE;
+    if (f != NULL) {
+	f->bufp = f->buffer;
+	f->left = BUFFILESIZE;
+    }
     return f;
 }
 
@@ -182,7 +185,7 @@ BufFileRead (BufFilePtr f, char *b, int n)
 }
 
 int
-BufFileWrite (BufFilePtr f, char *b, int n)
+BufFileWrite (BufFilePtr f, const char *b, int n)
 {
     int	    cnt;
     cnt = n;
