@@ -33,21 +33,19 @@ from The Open Group.
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#include "libxfontint.h"
 #include	<X11/fonts/fontmisc.h>
 #include	<X11/fonts/fontstruct.h>
 #include	<X11/fonts/fontxlfd.h>
 #include	<X11/fonts/fontutil.h>
+#include	<X11/fonts/fntfilst.h> /* for MAXFONTNAMELEN */
 #include	<X11/Xos.h>
 #include	<math.h>
 #include	<stdlib.h>
-#if defined(sony) && !defined(SYSTYPE_SYSV) && !defined(_SYSTYPE_SYSV)
-#define NO_LOCALE
-#endif
-#ifndef NO_LOCALE
 #include	<locale.h>
-#endif
 #include	<ctype.h>
 #include	<stdio.h>	/* for sprintf() */
+#include	"src/util/replace.h"
 
 static char *
 GetInt(char *ptr, int *val)
@@ -67,9 +65,7 @@ GetInt(char *ptr, int *val)
 #define maxchar(p) ((p).max_char_low + ((p).max_char_high << 8))
 
 
-#ifndef NO_LOCALE
 static struct lconv *locale = 0;
-#endif
 static const char *radix = ".", *plus = "+", *minus = "-";
 
 static char *
@@ -77,7 +73,6 @@ readreal(char *ptr, double *result)
 {
     char buffer[80], *p1, *p2;
 
-#ifndef NO_LOCALE
     /* Figure out what symbols apply in this locale */
 
     if (!locale)
@@ -90,7 +85,6 @@ readreal(char *ptr, double *result)
 	if (locale->negative_sign && *locale->negative_sign)
 	    minus = locale->negative_sign;
     }
-#endif
     /* Copy the first 80 chars of ptr into our local buffer, changing
        symbols as needed. */
     for (p1 = ptr, p2 = buffer;
@@ -113,13 +107,15 @@ readreal(char *ptr, double *result)
     return (p1 == buffer) ? (char *)0 : (ptr + (p1 - buffer));
 }
 
+#define XLFD_DOUBLE_TO_TEXT_BUF_SIZE 80
+
 static char *
 xlfd_double_to_text(double value, char *buffer, int space_required)
 {
     register char *p1;
     int ndigits, exponent;
+    const size_t buflen = XLFD_DOUBLE_TO_TEXT_BUF_SIZE;
 
-#ifndef NO_LOCALE
     if (!locale)
     {
 	locale = localeconv();
@@ -130,13 +126,12 @@ xlfd_double_to_text(double value, char *buffer, int space_required)
 	if (locale->negative_sign && *locale->negative_sign)
 	    minus = locale->negative_sign;
     }
-#endif
 
     if (space_required)
 	*buffer++ = ' ';
 
     /* Render the number using printf's idea of formatting */
-    sprintf(buffer, "%.*le", XLFD_NDIGITS, value);
+    snprintf(buffer, buflen, "%.*le", XLFD_NDIGITS, value);
 
     /* Find and read the exponent value */
     for (p1 = buffer + strlen(buffer);
@@ -145,22 +140,22 @@ xlfd_double_to_text(double value, char *buffer, int space_required)
     if (value == 0.0) exponent = 0;
 
     /* Figure out how many digits are significant */
-    while (p1 >= buffer && (!isdigit(*p1) || *p1 == '0')) p1--;
+    while (p1 >= buffer && (!isdigit((unsigned char)*p1) || *p1 == '0')) p1--;
     ndigits = 0;
-    while (p1 >= buffer) if (isdigit(*p1--)) ndigits++;
+    while (p1 >= buffer) if (isdigit((unsigned char)*p1--)) ndigits++;
 
     /* Figure out notation to use */
     if (exponent >= XLFD_NDIGITS || ndigits - exponent > XLFD_NDIGITS + 1)
     {
 	/* Scientific */
-	sprintf(buffer, "%.*le", ndigits - 1, value);
+	snprintf(buffer, buflen, "%.*le", ndigits - 1, value);
     }
     else
     {
 	/* Fixed */
 	ndigits -= exponent + 1;
 	if (ndigits < 0) ndigits = 0;
-	sprintf(buffer, "%.*lf", ndigits, value);
+	snprintf(buffer, buflen, "%.*lf", ndigits, value);
 	if (exponent < 0)
 	{
 	    p1 = buffer;
@@ -188,12 +183,7 @@ xlfd_round_double(double x)
       significant digits.  How do you round to n significant digits on
       a binary machine?  */
 
-#if defined(i386) || defined(__i386__) || \
-    defined(ia64) || defined(__ia64__) || \
-    defined(__alpha__) || defined(__alpha) || \
-    defined(__hppa__) || \
-    defined(__amd64__) || defined(__amd64) || \
-    defined(sgi)
+#ifdef HAVE_FLOAT_H
 #include <float.h>
 
 /* if we have IEEE 754 fp, we can round to binary digits... */
@@ -254,7 +244,7 @@ xlfd_round_double(double x)
    }
    else
 #endif
-#endif /* i386 || __i386__ */
+#endif /* HAVE_FLOAT_H */
     {
 	/*
 	 * If not IEEE 754:  Let printf() do it for you.
@@ -262,7 +252,7 @@ xlfd_round_double(double x)
 
 	char buffer[40];
 
-	sprintf(buffer, "%.*lg", XLFD_NDIGITS, x);
+	snprintf(buffer, sizeof(buffer), "%.*lg", XLFD_NDIGITS, x);
 	return atof(buffer);
     }
 }
@@ -278,7 +268,7 @@ GetMatrix(char *ptr, FontScalablePtr vals, int which)
 	matrix = vals->point_matrix;
     else return (char *)0;
 
-    while (isspace(*ptr)) ptr++;
+    while (isspace((unsigned char)*ptr)) ptr++;
     if (*ptr == '[')
     {
 	/* This is a matrix containing real numbers.  It would be nice
@@ -292,13 +282,13 @@ GetMatrix(char *ptr, FontScalablePtr vals, int which)
 	    (ptr = readreal(ptr, matrix + 2)) &&
 	    (ptr = readreal(ptr, matrix + 3)))
 	{
-	    while (isspace(*ptr)) ptr++;
+	    while (isspace((unsigned char)*ptr)) ptr++;
 	    if (*ptr != ']')
 		ptr = (char *)0;
 	    else
 	    {
 		ptr++;
-		while (isspace(*ptr)) ptr++;
+		while (isspace((unsigned char)*ptr)) ptr++;
 		if (*ptr == '-')
 		{
 		    if (which == POINTSIZE_MASK)
@@ -346,26 +336,28 @@ GetMatrix(char *ptr, FontScalablePtr vals, int which)
 
 
 static void
-append_ranges(char *fname, int nranges, fsRange *ranges)
+append_ranges(char *fname, size_t fnamelen, int nranges, fsRange *ranges)
 {
     if (nranges)
     {
         int i;
 
-        strcat(fname, "[");
+        strlcat(fname, "[", fnamelen);
         for (i = 0; i < nranges && strlen(fname) < 1010; i++)
         {
-	    if (i) strcat(fname, " ");
-	    sprintf(fname + strlen(fname), "%d",
-		    minchar(ranges[i]));
+	    size_t curlen;
+	    if (i) strlcat(fname, " ", fnamelen);
+	    curlen = strlen(fname);
+	    snprintf(fname + curlen, fnamelen - curlen, "%d",
+		     minchar(ranges[i]));
 	    if (ranges[i].min_char_low ==
 		ranges[i].max_char_low &&
 		ranges[i].min_char_high ==
 		ranges[i].max_char_high) continue;
-	    sprintf(fname + strlen(fname), "_%d",
-		    maxchar(ranges[i]));
+	    snprintf(fname + curlen, fnamelen - curlen, "_%d",
+		     maxchar(ranges[i]));
         }
-        strcat(fname, "]");
+        strlcat(fname, "]", fnamelen);
     }
 }
 
@@ -381,6 +373,8 @@ FontParseXLFDName(char *fname, FontScalablePtr vals, int subst)
     FontScalableRec tmpvals;
     char        replaceChar = '0';
     char        tmpBuf[1024];
+    size_t      tlen;
+    size_t      fnamelen = MAXFONTNAMELEN; /* assumed for now */
     int         spacingLen;
     int		l;
     char	*p;
@@ -438,8 +432,9 @@ FontParseXLFDName(char *fname, FontScalablePtr vals, int subst)
 	break;
     case FONT_XLFD_REPLACE_STAR:
 	replaceChar = '*';
+	/* FALLTHROUGH */
     case FONT_XLFD_REPLACE_ZERO:
-	strcpy(tmpBuf, ptr2);
+	strlcpy(tmpBuf, ptr2, sizeof(tmpBuf));
 	ptr5 = tmpBuf + (ptr5 - ptr2);
 	ptr3 = tmpBuf + (ptr3 - ptr2);
 	ptr2 = tmpBuf;
@@ -471,7 +466,7 @@ FontParseXLFDName(char *fname, FontScalablePtr vals, int subst)
 	    *ptr++ = '-';
 	}
 	*ptr++ = replaceChar;
-	strcpy(ptr, ptr5);
+	strlcpy(ptr, ptr5, fnamelen - (ptr - fname));
 	*vals = tmpvals;
 	break;
     case FONT_XLFD_REPLACE_VALUE:
@@ -507,68 +502,82 @@ FontParseXLFDName(char *fname, FontScalablePtr vals, int subst)
 
 	p = ptr1 + 1;				/* weight field */
 	l = strchr(p, '-') - p;
-	sprintf(tmpBuf, "%*.*s", l, l, p);
+	snprintf(tmpBuf, sizeof(tmpBuf), "%*.*s", l, l, p);
 
 	p += l + 1;				/* slant field */
 	l = strchr(p, '-') - p;
-	sprintf(tmpBuf + strlen(tmpBuf), "-%*.*s", l, l, p);
+	tlen = strlen(tmpBuf);
+	snprintf(tmpBuf + tlen, sizeof(tmpBuf) - tlen, "-%*.*s", l, l, p);
 
 	p += l + 1;				/* setwidth_name */
 	l = strchr(p, '-') - p;
-	sprintf(tmpBuf + strlen(tmpBuf), "-%*.*s", l, l, p);
+	tlen = strlen(tmpBuf);
+	snprintf(tmpBuf + tlen, sizeof(tmpBuf) - tlen, "-%*.*s", l, l, p);
 
 	p += l + 1;				/* add_style_name field */
 	l = strchr(p, '-') - p;
-	sprintf(tmpBuf + strlen(tmpBuf), "-%*.*s", l, l, p);
+	tlen = strlen(tmpBuf);
+	snprintf(tmpBuf + tlen, sizeof(tmpBuf) - tlen, "-%*.*s", l, l, p);
 
-	strcat(tmpBuf, "-");
+	strlcat(tmpBuf, "-", sizeof(tmpBuf));
 	if ((tmpvals.values_supplied & PIXELSIZE_MASK) == PIXELSIZE_ARRAY)
 	{
-	    char buffer[80];
-	    strcat(tmpBuf, "[");
-	    strcat(tmpBuf, xlfd_double_to_text(tmpvals.pixel_matrix[0],
-		   buffer, 0));
-	    strcat(tmpBuf, xlfd_double_to_text(tmpvals.pixel_matrix[1],
-		   buffer, 1));
-	    strcat(tmpBuf, xlfd_double_to_text(tmpvals.pixel_matrix[2],
-		   buffer, 1));
-	    strcat(tmpBuf, xlfd_double_to_text(tmpvals.pixel_matrix[3],
-		   buffer, 1));
-	    strcat(tmpBuf, "]");
+	    char buffer[XLFD_DOUBLE_TO_TEXT_BUF_SIZE];
+	    strlcat(tmpBuf, "[", sizeof(tmpBuf));
+	    strlcat(tmpBuf,
+		    xlfd_double_to_text(tmpvals.pixel_matrix[0], buffer, 0),
+		    sizeof(tmpBuf));
+	    strlcat(tmpBuf,
+		    xlfd_double_to_text(tmpvals.pixel_matrix[1], buffer, 1),
+		    sizeof(tmpBuf));
+	    strlcat(tmpBuf,
+		    xlfd_double_to_text(tmpvals.pixel_matrix[2], buffer, 1),
+		    sizeof(tmpBuf));
+	    strlcat(tmpBuf,
+		    xlfd_double_to_text(tmpvals.pixel_matrix[3], buffer, 1),
+		    sizeof(tmpBuf));
+	    strlcat(tmpBuf, "]", sizeof(tmpBuf));
 	}
 	else
 	{
-	    sprintf(tmpBuf + strlen(tmpBuf), "%d",
-		    (int)(tmpvals.pixel_matrix[3] + .5));
+	    tlen = strlen(tmpBuf);
+	    snprintf(tmpBuf + tlen, sizeof(tmpBuf) - tlen, "%d",
+		     (int)(tmpvals.pixel_matrix[3] + .5));
 	}
-	strcat(tmpBuf, "-");
+	strlcat(tmpBuf, "-", sizeof(tmpBuf));
 	if ((tmpvals.values_supplied & POINTSIZE_MASK) == POINTSIZE_ARRAY)
 	{
-	    char buffer[80];
-	    strcat(tmpBuf, "[");
-	    strcat(tmpBuf, xlfd_double_to_text(tmpvals.point_matrix[0],
-		   buffer, 0));
-	    strcat(tmpBuf, xlfd_double_to_text(tmpvals.point_matrix[1],
-		   buffer, 1));
-	    strcat(tmpBuf, xlfd_double_to_text(tmpvals.point_matrix[2],
-		   buffer, 1));
-	    strcat(tmpBuf, xlfd_double_to_text(tmpvals.point_matrix[3],
-		   buffer, 1));
-	    strcat(tmpBuf, "]");
+	    char buffer[XLFD_DOUBLE_TO_TEXT_BUF_SIZE];
+	    strlcat(tmpBuf, "[", sizeof(tmpBuf));
+	    strlcat(tmpBuf,
+		    xlfd_double_to_text(tmpvals.point_matrix[0], buffer, 0),
+		    sizeof(tmpBuf));
+	    strlcat(tmpBuf,
+		    xlfd_double_to_text(tmpvals.point_matrix[1], buffer, 1),
+		    sizeof(tmpBuf));
+	    strlcat(tmpBuf,
+		    xlfd_double_to_text(tmpvals.point_matrix[2], buffer, 1),
+		    sizeof(tmpBuf));
+	    strlcat(tmpBuf,
+		    xlfd_double_to_text(tmpvals.point_matrix[3], buffer, 1),
+		    sizeof(tmpBuf));
+	    strlcat(tmpBuf, "]", sizeof(tmpBuf));
 	}
 	else
 	{
-	    sprintf(tmpBuf + strlen(tmpBuf), "%d",
-		    (int)(tmpvals.point_matrix[3] * 10.0 + .5));
+	    tlen = strlen(tmpBuf);
+	    snprintf(tmpBuf + tlen, sizeof(tmpBuf) - tlen, "%d",
+		     (int)(tmpvals.point_matrix[3] * 10.0 + .5));
 	}
-	sprintf(tmpBuf + strlen(tmpBuf), "-%d-%d%*.*s%d%s",
-		tmpvals.x, tmpvals.y,
-		spacingLen, spacingLen, ptr3, tmpvals.width, ptr5);
-	strcpy(ptr1 + 1, tmpBuf);
+	tlen = strlen(tmpBuf);
+	snprintf(tmpBuf + tlen, sizeof(tmpBuf) - tlen, "-%d-%d%*.*s%d%s",
+		 tmpvals.x, tmpvals.y,
+		 spacingLen, spacingLen, ptr3, tmpvals.width, ptr5);
+	strlcpy(ptr1 + 1, tmpBuf, fnamelen - (ptr1 - fname));
 	if ((vals->values_supplied & CHARSUBSET_SPECIFIED) && !vals->nranges)
-	    strcat(fname, "[]");
+	    strlcat(fname, "[]", fnamelen);
 	else
-	    append_ranges(fname, vals->nranges, vals->ranges);
+	    append_ranges(fname, fnamelen, vals->nranges, vals->ranges);
 	break;
     }
     return TRUE;
